/**
 * Copyright (C) 2001-2002
 * - France Telecom R&D
 * - Laboratoire Logiciels, Systemes, Reseaux - UMR 5526, CNRS-INPG-UJF
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors:
 *
 */

package org.objectweb.perseus.concurrency.dbdelegate;

import org.objectweb.perseus.concurrency.api.ConcurrencyException;
import org.objectweb.perseus.concurrency.api.ConcurrencyManager;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author P. Dechamboux
 */
public abstract class DbDelegateConcurrencyManager
        implements ConcurrencyManager, BindingController, LifeCycleController {
	private String state = LifeCycleController.STOPPED;

    protected Logger logger = null;

	public DbDelegateConcurrencyManager() throws ConcurrencyException {
	}

	/**
	 * @param ctx is the context using the resource
	 * @param resource is the resource
	 * @param thinLock uses to take a thin lock
	 * @return the resource state to use
	 */
	protected abstract Object getState(Object ctx,
									   Object resource,
									   Object thinLock) throws ConcurrencyException;


    //IMPLEMENTATION OF THE UserBindingControler INTERFACE //
    //------------------------------------------------------//

    public String[] listFc() {
        return new String[] {};
    }

    public Object lookupFc(String s) throws NoSuchInterfaceException {
//FIXME:		throw new NoSuchInterfaceException(s);
		return null;
    }

	public void bindFc(String s, Object o) throws IllegalBindingException,
			NoSuchInterfaceException {
		try {
		} catch (ClassCastException e) {
			throw new IllegalBindingException(s + ":"  + e.getMessage());
		}
//FIXME:		throw new NoSuchInterfaceException(s);
	}

    public void unbindFc(String s) throws NoSuchInterfaceException {
//FIXME:		throw new NoSuchInterfaceException(s);
    }

	//IMPLEMENTATION OF THE LifeCycleController INTERFACE //
	//----------------------------------------------------//
	public String getFcState() {
		return state;
	}

	public void startFc() throws IllegalLifeCycleException {
		if (state == LifeCycleController.STARTED) return;
		state = LifeCycleController.STARTED;
	}

	public void stopFc() throws IllegalLifeCycleException {
		if (state == LifeCycleController.STOPPED) return;
		state = LifeCycleController.STOPPED;
	}

	//IMPLEMENTATION OF THE ConcurrencyManager INTERFACE //
	//---------------------------------------------------//

	public void begin(Object ctx) {
	}

	public boolean validate(Object ctx) {
		return true;
	}

	public abstract void finalize(Object ctx);

	public void abort(Object ctx) {
		finalize(ctx);
	}

	public Object readIntention(Object ctx, Object resourceId, Object hints)
			throws ConcurrencyException {
		return getState(ctx, resourceId, hints);
	}

	public Object writeIntention(Object ctx, Object resourceId, Object hints)
			throws ConcurrencyException {
		return getState(ctx, resourceId, hints);
	}
}
